import { Injectable, NotFoundException } from '@nestjs/common';
import { CreateEmprestimoDto } from './dto/create-emprestimo.dto';
import { UpdateEmprestimoDto } from './dto/update-emprestimo.dto';
import { PrismaService } from 'src/prisma/prisma.service';
import { DateTime } from 'luxon';
import { LivroService } from 'src/livro/livro.service';
import { FinalizarEmprestimoDto } from './dto/finalizar-emprestimo';
import { FilterEmprestimoDto } from './dto/filter-emprestimo';
import { UserService } from 'src/user/user.service';
import { EmptyError } from 'rxjs';

@Injectable()
export class EmprestimoService {
  constructor(private prisma: PrismaService, private livro: LivroService, private readonly userService: UserService) { }

  async create(data: CreateEmprestimoDto) {
    await this.verifLivro(data.livro_id)
    data.data_emprestimo = await this.formatDatas(data.data_emprestimo)
    if (data.data_prazo)
      data.data_prazo = await this.formatDatas(data.data_prazo)
    if (data.data_entrega)
      data.data_entrega = await this.formatDatas(data.data_entrega)

    data.historico = await this.userService.criar_historico(data.user_id, data)

    const emprestimo = await this.prisma.emprestimo.create({ data })
    if (emprestimo) await this.livro.updateStatus(data.livro_id, 2)
    return emprestimo
  }

  async findAll(establishment_id: string) {
    return this.prisma.emprestimo.findMany({
      where: {
        establishment_id: establishment_id,
        NOT: {
          status: 0
        },

      },
      include: {
        livro: true
      },
      orderBy: {
        id: "desc"
      }
    });
  }

  async search(establishment_id: string, filter: FilterEmprestimoDto) {
    const status = filter.status
    const text = filter.text
    const type = filter.type

    if (type == "livro") {

      return this.prisma.emprestimo.findMany({
        where: {
          establishment_id: establishment_id,
          status: status,
          livro: {
            titulo: {
              contains: text,
              mode: 'insensitive',
            }
          }
        },
        include: {
          livro: true,
          aluno: true
        },

        orderBy: {
          id: "desc"
        }
      });

    }

    if (type == "aluno") {

      return this.prisma.emprestimo.findMany({
        where: {
          establishment_id: establishment_id,
          status: status,
          aluno: {
            name: {
              contains: text,
              mode: 'insensitive',
            }
          }
        },
        include: {
          livro: true,
          aluno: true
        },

        orderBy: {
          id: "desc"
        }
      });

    }

    if (text != "" || text != null) {

      return this.prisma.emprestimo.findMany({
        where: {
          establishment_id: establishment_id,
          status: status,
          OR: [
            {
              aluno: {
                name: {
                  contains: text,
                  mode: 'insensitive',
                }
              }
            },
            {
              livro: {
                titulo: {
                  contains: text,
                  mode: 'insensitive',
                }
              }
            }
          ]

        },
        include: {
          livro: true,
          aluno: true
        },

        orderBy: {
          id: "desc"
        }
      });

    }

    return this.prisma.emprestimo.findMany({
      where: {
        establishment_id: establishment_id,
        status: status,
      },
      include: {
        livro: true,
        aluno: true
      },

      orderBy: {
        id: "desc"
      }
    });





  }

  async findOne(id: string) {
    try {
      return await this.prisma.emprestimo.findUnique({
        where: {
          id
        }
      })
    } catch (error) {
      throw new NotFoundException(`Nao encontramos emprestimos com este id  ${id}`);
    }
  }

  async update(id: string, data: UpdateEmprestimoDto, user_id: string) {

    const emprestimo = await this.ifExist(id)

    //caso em que update troca livro
    if (emprestimo.livro_id != data.livro_id) await this.trocarLivro(emprestimo.livro_id, data.status, data.livro_id)

    console.log(data)

    //formata datas 
    if (data.data_prazo) { data.data_prazo = await this.formatDatas(data.data_prazo) }

    if (data.data_entrega) { data.data_entrega = await this.formatDatas(data.data_entrega) }

    if (data.data_emprestimo) { data.data_emprestimo = await this.formatDatas(data.data_emprestimo) }

    if (data.status == 2) await this.livro.updateStatus(data.livro_id, 1) // se o emprestimo estiver sendo finalizado, livro volta a ficar disponivel(status 1)

    //add historico
    const historico_atual = JSON.stringify(emprestimo.historico)
    delete (emprestimo.historico)
    delete (emprestimo.id)
    data.historico = await this.userService.update_historico(user_id, emprestimo, data, historico_atual, "UPDATE")

    return await this.prisma.emprestimo.update({
      data,
      where: {
        id
      }
    })
  }

  async finalizar(data: FinalizarEmprestimoDto, id: string, user_id) {

    data.status = 2

    const emprestimo = await this.ifExist(id)

    //formata data
    if (data.data_entrega) { data.data_entrega = await this.formatDatas(data.data_entrega) }

    //muda status livro
    await this.livro.updateStatus(emprestimo.livro_id, 1)

    const historico_atual = JSON.stringify(emprestimo.historico)
    delete (emprestimo.historico)
    delete (emprestimo.id)
    data.historico = await this.userService.update_historico(user_id, emprestimo, data, historico_atual, "FINALIZAR")

    return await this.prisma.emprestimo.update({
      data,
      where: {
        id
      }
    })

  }

  async delete(id: string, user_id: string) {

    await this.ifExist(id)

    const emprestimo = await this.findOne(id)
    if (emprestimo.status == 1) {// se o emprestimo estiver pendente ainda e ....
      const livro = await this.livro.findOne(emprestimo.livro_id)
      if (livro.status == 2) { // ... se o livro estiver como ocupado...
        await this.livro.updateStatus(emprestimo.livro_id, 1) // ... livro volta a ficar disponivel
      }
    }

    //add historico
    const old_historico = JSON.stringify(emprestimo.historico)
    delete (emprestimo.historico)
    delete (emprestimo.id)

    const data = {
      status: 0,
      historico: await this.userService.update_historico(user_id, emprestimo, { status: 0 }, old_historico, 'DELETE')
    }

    return this.prisma.emprestimo.update({
      data,
      where: {
        id
      }
    })
  }

  async ifExist(id: string) {
    const emprestimo = await this.findOne(id)
    if (!(emprestimo)) {
      throw new NotFoundException(`Nao encontramos emprestimo com este id  ${id}`);
    }
    return emprestimo
  }

  async verifLivro(id: string) {
    const livro = await this.livro.findOne(id)
    if (livro.status != 1) {
      throw new NotFoundException(`Este livro não pode ser emprestado, pois não esta com status ativo. Verificar se já não existe emprestimo!`);
    }
  }

  async trocarLivro(livro_id: string, status_emprestimo: number, new_livro_id: string) {
    await this.verifLivro(new_livro_id)
    await this.livro.updateStatus(livro_id, 1) // livro antigo fica disponível
    if (status_emprestimo != 2) await this.livro.updateStatus(livro_id, 2) // livro novo indisponível
  }

  async formatDatas(date: DateTime) {
    // return date
    const dateTime = DateTime.fromISO(date);
    const dataFormatada = dateTime.toISO();
    return dataFormatada

  }

  //total geral por ano
  async getEmprestimosCompanyAno(company_id: string, ano: string) {
    return this.prisma.emprestimo.findMany({
      where: {
        status: 2,
        establishment: {
          company_id: company_id
        },
        data_emprestimo: {
          gte: new Date(`${ano}-01-01T00:00:00.000Z`),
          lt: new Date(`${parseInt(ano) + 1}-01-01T00:00:00.000Z`),
        },
      },
      include: {
        establishment: {
          select: {
            company_id: true
          }
        }
      },
      orderBy: {
        id: "desc"
      }
    });
  }

  //total geral por ano e mes
  async getEmprestimosPorAnoEContagem(company_id: string, ano: string) {
    const emprestimosPorAno = await this.prisma.emprestimo.findMany({
      where: {
        status: 2,
        establishment: {
          company_id: company_id,
        },
        data_emprestimo: {
          gte: new Date(`${ano}-01-01T00:00:00.000Z`), 
          lt: new Date(`${parseInt(ano) + 1}-01-01T00:00:00.000Z`),
        },
      },
      orderBy:{
        data_emprestimo:"asc"
      }
    });
  
    const emprestimosPorMes = {};
    const meses = ["Enero","Febrero","Marzo","Abril","Mayo","Junio","Julio","Agosto","Septiembre","Octubre","Noviembre","Diciembre"]
    const lista_meses = []
    emprestimosPorAno.forEach((emprestimo) => {
      const data = new Date(emprestimo.data_emprestimo)
      const mes = data.getMonth() + 1
      

      if (!emprestimosPorMes[mes]) {
        emprestimosPorMes[mes] = 0;
        lista_meses.push(meses[data.getMonth()])
      }
      emprestimosPorMes[mes]++;
    });
    // const keys = Object.keys(emprestimosPorMes);
    const num = Object.values(emprestimosPorMes);
    console.log(num)
    console.log(lista_meses)
  
    const dados = {
      meses: lista_meses,
      numeros: num
    }
    return dados;
  }







}

