import { Injectable, NotFoundException } from '@nestjs/common';
import { CreateLivroDto } from './dto/create-livro.dto';
import { UpdateLivroDto } from './dto/update-livro.dto';
import { PrismaService } from 'src/prisma/prisma.service';
import { DateTime } from 'luxon';
import { Company } from 'src/decorators/company_decorator';
@Injectable()
export class LivroService {

  constructor(private prisma: PrismaService) { }

  async create(data: CreateLivroDto, estab_id: string) {
    data.establishment_id = estab_id
    if (data.data_aquisicao) data.data_aquisicao = await this.formatDatas(data.data_aquisicao)
    return this.prisma.livro.create({
      data
    });

  } 

  async findAllEstabelecimento(establishment_id: string) {
    return this.prisma.livro.findMany({
      where: {
        establishment_id,
        NOT: {
          status: 0
        },
      },
      include: {
        categoria: {
          select: {
            nome: true,
            id: true
          },
        },
        Livro_scdd: {
          where:{
            status:1
          },
          include: {
            scdd: {
              select: {
                name: true,
                id: true,
                descricao: true
              }
            }
          }
        },
        Livro_autor:{
          where:{
            status:1
          },
          include:{
            Autor:{
              select:{
                nome:true,
                id: true
              }
            },
            Cutter:{
              select:{
                name:true,
                id: true
              }

            }
          }
        }
      },
      orderBy: {
        id: "desc"
      }

    });
  }

  async findAll(text: string) {
    const data = this.prisma.livro.findMany({
      where: {

        OR: [
          {
            titulo: {
              contains: text,
              mode: 'insensitive'
            },
          },
          {
            editorial: {
              contains: text,
              mode: 'insensitive'
            },
          },
          {
            categoria: {
              nome: {
                contains: text,
                mode: 'insensitive'

              }
            },
          },


        ],
        AND: {
          NOT: {
            status: 0
          }
        }

      },
      include: {

        categoria: {
          select: {
            nome: true,
            id: true
          },
        },
        Livro_autor: {
          include: {
            Autor: {
              select: {
                nome: true
              }
            },
            Cutter: {
              select: {
                name: true
              }
            }
          }
        },
        establishment: {
          // select: {
          //   name: true,
          // },
          include: {
            company: {
              select: {
                sigla: true
              }
            }
          }
        }
      },

    });

    // return data

    return (await data).map((element) => {
      return {
        company: element.establishment.company.sigla,
        establisment: element.establishment.name,
        titulo: element.titulo,
        editorial: element.editorial,
        edicao: element.edicao,
        ano: element.ano,
        status: element.status,
        autores: element.Livro_autor.map((e) => {
          return e.Autor.nome + " "
        }).flat()

      }
    })

  }

  async search(filter: UpdateLivroDto, establishment_id: string) {
    console.log(establishment_id)
    const categoria = filter.categoria_id
    const titulo = filter.titulo
    if (('categoria_id' in filter) && ('titulo' in filter)) {

      return this.prisma.livro.findMany({
        where: {
          categoria_id: filter.categoria_id,
          establishment_id,
          titulo: {
            contains: filter.titulo,
            mode: 'insensitive',
          },
          NOT: {
            status: 0
          },
        },
        include: {
          categoria: {
            select: {
              nome: true,
              id: true
            },
          }
        },
      });
    }

    if ('titulo' in filter) {
      console.log("Só titulo")
      return this.prisma.livro.findMany({
        where: {
          establishment_id,
          OR: [
            {
              titulo: {
                contains: titulo,
                mode: 'insensitive'
              },
            },
            { codigo: { contains: titulo } },
          ],


          NOT: {
            status: 0
          },
        },
        include: {

          categoria: {
            select: {
              nome: true,
              id: true
            },
          },
        },
      });
    }

    if ('categoria_id' in filter) {

      return this.prisma.livro.findMany({
        where: {
          categoria_id: categoria,
          establishment_id,
          NOT: {
            status: 0
          },
        },
        include: {

          categoria: {
            select: {
              nome: true,
              id: true
            },
          },
        },
      });
    }
  }

  async findStatus(establishment_id: string, status: number) {
    // return "ok"
    return this.prisma.livro.findMany({
      where: {
        establishment_id,
        status: status
      },
      include: {
        categoria: {
          select: {
            nome: true,
            id: true
          },
        },
      },

    });
  }

  async findOne(id: string) {
    try {
      return await this.prisma.livro.findUnique({
        where: {
          id
        }
      })
    } catch (error) {
      throw new NotFoundException(`Nao existe Livro com este id  ${id}`);
    }
  }

  async update(id: string, data: UpdateLivroDto) {
    await this.ifExist(id)
    if (data.data_aquisicao) data.data_aquisicao = await this.formatDatas(data.data_aquisicao)
    return await this.prisma.livro.update({
      data,
      where: {
        id
      }
    })
  }
  async updateStatus(id: string, status: number) {
    await this.ifExist(id)

    return await this.prisma.livro.update({
      data: {
        status: status
      },
      where: {
        id
      }
    })
  }

  async delete(id: string) {

    await this.ifExist(id)

    return this.prisma.livro.update({
      data: {
        status: 0
      },
      where: {
        id
      }
    })
  }

  async ifExist(id: string) {
    if (!(await this.findOne(id))) {
      throw new NotFoundException(`Nao existe livro com este id  ${id}`);
    }
  }

  async formatDatas(date: Date) {
    const dateTime = await DateTime.fromISO(date);
    return DateTime.utc().toISO(dateTime)
  }
}
