import { Injectable, NotFoundException } from '@nestjs/common';
import { PrismaService } from 'src/prisma/prisma.service'
// import { User, Prisma } from '@prisma/client';
import { DateTime } from 'luxon';
import { CreateUserDto } from './dto/create-user.dto';
import { UpdateUserDto } from './dto/update-user.dto';
import * as bcrypt from 'bcrypt';
import { BadRequestException } from '@nestjs/common';

@Injectable()
export class UserService {

  constructor(private prisma: PrismaService) { }

  async create(data: CreateUserDto) {

    if (!data.status) data.status = 1

    const saltRounds = 10; // Define o número de rounds de hashing
    const hashedPassword = await bcrypt.hash(data.password, saltRounds);
    data.password = hashedPassword

    try {
      return await this.prisma.user.create({
        data
      });

    } catch (error) {
      return await this.tratar_erros_banco(error)

    }


  }

  async users() {
    return this.prisma.user.findMany({
      where: {
        status: 1
      }
    });
  }

  async userId(id: string) {

    return this.prisma.user.findUnique({
      where: {
        id
      }
    })
  }

  async update(id: string, data: UpdateUserDto) {

    await this.ifExist(id)

    return this.prisma.user.update({
      data,
      where: {
        id
      }
    })
  }

  async delete(id: string) {

    await this.ifExist(id)

    return this.prisma.user.update({
      data: {
        status: 0
      },
      where: {
        id
      }
    })
  }

  async getRoles(id: string) {

    const userPermissions = await this.prisma.user.findUnique({
      where: { id: id },
      include: {
        User_has_role: {
          include: {
            role: {
              include: {
                Permission_has_role: {
                  include: {
                    permission: {
                      select: {
                        name: true
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    });

    const permissions = userPermissions.User_has_role.map(
      (userRole) => userRole.role.Permission_has_role.map(
        (permissionRole) => permissionRole.permission.name
      )
    ).flat();

    return permissions
  }

  async getPermisions(id: string) {

    const userPermissions = await this.prisma.user.findUnique({
      where: { id: id },
      include: {
        User_has_role: {
          include: {
            role: {
              include: {
                Permission_has_role: {
                  include: {
                    permission: {
                      select: {
                        name: true
                      }
                    }
                  }
                },
                establishment: {
                  select: {
                    name: true,
                    company_id: true
                  },
                  // include:{
                  //   company:true
                  // }
                },
              }

            }
          }
        }
      }
    });

    // const permissions = userPermissions.User_has_role.map(
    //   (userRole) => userRole.role.Permission_has_role.map(
    //     (permissionRole) => permissionRole.permission.name
    //   ) 
    // ).flat();

    const permissions = userPermissions.User_has_role.map(function (elemento) {
      return {
        establishment_id: elemento.role.establishment_id,
        establishment: elemento.role.establishment.name,
        company_id: elemento.role.establishment.company_id,
        role: elemento.role.name,
        permissions: elemento.role.Permission_has_role.map((permisionRole) => permisionRole.permission.name).flat()
      }
    }).flat()

    return permissions
  }

  async ifExist(id: string) {
    if (!(await this.userId(id))) {
      throw new NotFoundException(`Nao encontramos user com este id  ${id}`);
    }
  }

  tratar_erros_banco(error) {
    if (error.code === 'P2002') {
      if (error.meta.target == "users_email_key") throw new BadRequestException("Já existe um usuario com este EMAIL")
      if (error.meta.target == "users_username_key") throw new BadRequestException("Já existe um usuario com este USERNAME")
    } else {
      return error
    }
  }

  async criar_historico(user_id: string, data: object) {
    const user = await this.userId(user_id)
    const array = []
    const date = DateTime.local({ zone: "America/Asuncion" })
    // console.log(date)
    // console.log(date.toFormat('dd/MM/yyyy HH:mm:ss'))
    const historico = {
      type: "CREATE",
      user_id: user_id,
      name_user: user.name,
      old_value: null,
      new_value: data,
      date: date.toFormat('dd/MM/yyyy HH:mm:ss')
    }
    array.push(historico)
    return JSON.stringify(array)
  }

  async update_historico(user_id: string, old_value: object, new_value: object, historico: string, type: string) {
    const date = DateTime.local({ zone: "America/Asuncion" })
    const user = await this.userId(user_id)
    const array_historico = JSON.parse(JSON.parse(historico))
    console.log(array_historico)
    const new_historico = {
      type: type,
      user_id: user_id,
      name_user: user.name,
      old_value: old_value,
      new_value: new_value,
      date: date.toFormat('dd/MM/yyyy HH:mm:ss')
    }
    array_historico.push(new_historico)
    console.log(array_historico)
    return JSON.stringify(array_historico)
  }




}