import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common';
import { CreateUsuarioDto } from './dto/create-usuario.dto';
import { UpdateUsuarioDto } from './dto/update-usuario.dto';
import { PrismaService } from 'src/prisma/prisma.service';
import { FilterUsuarioDto } from './dto/filter-usuario.dto';
import { ImportAlunosDto } from './dto/import-alunos.dto';

@Injectable()
export class UsuarioService {
  constructor(private prisma: PrismaService) { }
  async create(data: CreateUsuarioDto) {
    if (data.cpf) await this.ifExistCPF(data.cpf)
    if (data.usuarios_id) await this.ifExistUsuarioID(data.usuarios_id)
    if (!data.status) data.status = 1

    try {
      return await this.prisma.usuario.create({
        data
      });

    } catch (error) {
      return await this.tratar_erros_banco(error)

    }

  }

  async findAll() {
    return this.prisma.usuario.findMany({
      where: {
        status: 1
      },
      include: {
        User: {
          select: {
            username: true,
            id: true
          }
        }
      }
    });
  }

  async findPerfil(perfil_id: string) {
    return this.prisma.usuario.findMany({
      where: {
        status: 1,
        perfil_id: perfil_id
      },
      include: {
        User: {
          select: {
            username: true,
            id: true
          }
        }
      },
      orderBy: {
        id: "asc"
      },
      take: 50
    });
  }


  async search(perfil_id: string, dados: FilterUsuarioDto) {
    if (dados.type == "rg") {
      return this.prisma.usuario.findMany({
        where: {
          status: 1,
          perfil_id: perfil_id,
          rg_ci: {
            contains: dados.text,
            mode: 'insensitive'
          }
        },
        include: {
          User: {
            select: {
              username: true,
              id: true
            }
          }
        }
      });
    }
    else {
      return this.prisma.usuario.findMany({
        where: {
          status: 1,
          perfil_id: perfil_id,
          name: {
            contains: dados.text,
            mode: 'insensitive'
          }
        },
        include: {
          User: {
            select: {
              username: true,
              id: true
            }
          }
        }
      });
    }
  }

  async usuarioId(id: string) {

    return this.prisma.usuario.findUnique({
      where: {
        id
      }
    })
  }

  async update(id: string, data: UpdateUsuarioDto) {

    await this.ifExist(id)

    return this.prisma.usuario.update({
      data,
      where: {
        id
      }
    })

  }

  async delete(id: string) {

    await this.ifExist(id)

    return this.prisma.usuario.update({
      data: {
        status: 0
      },
      where: {
        id
      }
    })
  }

  async ifExist(id: string) {
    if (!(await this.usuarioId(id))) {
      throw new NotFoundException(`Nao encontramos USUARIO com este id  ${id}`);
    }
  }

  tratar_erros_banco(error) {
    if (error.code === 'P2002') {
      if (error.meta.target == "users_email_key") throw new BadRequestException("Já existe um usuario com este EMAIL")
      // if (error.meta.target == "users_username_key") throw new BadRequestException("Já existe um usuario com este USERNAME")
    } else {
      console.log("erro", error)
      return error
    }
  }

  async ifExistCPF(cpf: string) {
    const user = await this.prisma.usuario.findFirst({
      where: {
        cpf: cpf
      }
    })
    if (user) {
      throw new NotFoundException(`Já existe um usuario com este CPF ${cpf}`);
    }

  }

  // create para importacao de alunos
  async createImport(data: ImportAlunosDto) {
    const verif = await this.ifExistUsuarioID(data.usuarios_id)
    if (verif) {

      if (data.cpf == null) delete (data.cpf)
      if (data.email == null) delete (data.email)
      if (data.rg_ci == null) delete (data.rg_ci)
      if (data.telefone == null) delete (data.telefone)

      try {
        return await this.prisma.usuario.create({
          data
        });

      } catch (error) {
        return await this.tratar_erros_banco(error)

      }
    } else {
      console.log(`${data.name} não importado. usuarios_id: ${data.usuarios_id}, catraca: ${data.catraca}`)
    }


  }

  // create para importacao de alunos
  async createImportMany(data: ImportAlunosDto[]) {
    // const verif = await this.ifExistUsuarioID(data.usuarios_id)

    data.forEach(element => {
      if (element.cpf == null) delete (element.cpf)
      if (element.email == null) delete (element.email)
      if (element.rg_ci == null) delete (element.rg_ci)
      if (element.telefone == null) delete (element.telefone)
    });


    try {
      return await this.prisma.usuario.createMany({
        data
      });

    } catch (error) {
      return await this.tratar_erros_banco(error)

    }



  }


  async ifExistUsuarioID(usuarios_id: number) {
    const user = await this.prisma.usuario.findFirst({
      where: {
        usuarios_id
      }
    })
    if (user) {
      console.log(`Já existe um usuario com este numero de usario(usuarios_id) ${usuarios_id}`);
      return false
      // throw new NotFoundException(`Já existe um usuario com este numero de usario(usuarios_id) ${usuarios_id}`);

    }
    return true

  }

}
